# Copyright (c) 2008, Christopher Cowart and contributors
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions 
# are met:
# * Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright 
#   notice, this list of conditions and the following disclaimer in the 
#   documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: confmancommon.sh.in 586 2011-12-23 08:42:07Z blee $
#
# This file is a global include for all confman-based utilities. It
# provides the common interfaces for loading the shell library, setting
# up the use of syslog, and reading configurations.

# Some global definitions. Defaults that can be overriden by options.
DEFAULTS="/usr/local/etc/defaults/confman.conf"
UCONF="${HOME}/.confmanrc"
MYNAME=`basename $0`
if [ `id -u` != "0" ]; then
    SUDO="/usr/local/bin/sudo"
fi

# First things first. Check to see if the logging fd is open. If it's not,
# we re-spawn confman inside a pipe.
if (true >&5) 2>/dev/null ; then
	exec 4>&2 2>&5 >&3
else
	exec 3>&1
	exec $0 "$@" 5>&1 1>&3 | logger -t "$MYNAME[$$]: $USER" -s
	exit ${PIPESTATUS[0]}
fi

# After the fork, let's MY_PID so subshells can throw errors that will
# actually kill the parent
MY_PID="$$"

# Suck in the default configuration
if [ -f "$DEFAULTS" ] ; then
    . "$DEFAULTS"
else
	echo "$DEFAULTS couldn't be located" >&2
	exit 1
fi

# Get the global config
if [ -f "$GCONF" ] ; then 
	. "$GCONF"
else
	echo "Global config file couldn't be located" >&2
	exit 1
fi

if [ -z "$UNAME" ] ; then
    UNAME=`uname`
fi
case $UNAME in
    FreeBSD)
    install_cmd="install -Sp" 
    mktemp_file="mktemp -t confman"
    mktemp_dir="mktemp -d -t confman"
    sed_cmd="sed -E"
    function sed_i_cmd {
        ${sed_cmd} -i '' "$@"
    }
    stat_cmd="stat -f"
    stat_opts="mode=%Mp%Lp owner=%Su group=%Sg"
    readlink_cmd="/usr/local/bin/greadlink"
    ;;
    Linux)
    install_cmd="install -p"
    mktemp_file="mktemp -t confman.XXXXXX"
    mktemp_dir="mktemp -t -d confman.XXXXXX"
    sed_cmd="sed -r"
    function sed_i_cmd {
        ${sed_cmd} -i "$@"
    }
    stat_cmd="stat -c"
    stat_opts="mode=%a owner=%U group=%G"
    readlink_cmd="readlink"
    ;;
    *)
    install_cmd="install -Sp" 
    mktemp_file="mktemp -t confman"
    mktemp_dir="mktemp -d -t confman"
    sed_cmd="sed -E"
    function sed_i_cmd {
        ${sed_cmd} -i '' "$@"
    }
    stat_cmd="stat -f"
    stat_opts="mode=%Mp%Lp owner=%Su group=%Sg"
    readlink_cmd="/usr/local/bin/greadlink"
    ;;
esac

# Things that don't vary by OS
fetch_cmd="/usr/local/bin/wget -q -O"

# Create a base temporary directory for this process
export TMPDIR=`$mktemp_dir`

# Get the user config
if [ -f "$UCONF" ] ; then
	. "$UCONF"
fi

# Set debugging:
if [ -z $DEBUG ] ; then
	DEBUG="false"
fi

# Now, we source the library
if [ -f $REPO_LIBRARY ] ; then
	. $REPO_LIBRARY
else
	echo "Couldn't source the confman shell library" >&2
	exit 1
fi

# Source the filedb library
if [ -r "${FILEDB_LIBRARY}" ]; then
    . "${FILEDB_LIBRARY}"
else
    echo "Could not load filedb library: ${FILEDB_LIBRARY}" >&2
    exit 1
fi

# Create and/or fix permissions on our local state directory
if ! [ -d "$REPO_DB" ] ; then
    $SUDO $install_cmd -d -o root -g $CONF_GROUP -m 2770 "$REPO_DB"
fi
find $REPO_DB -not -group $CONF_GROUP -exec $SUDO chgrp $CONF_GROUP {} \; \
    -exec chmod g+r {} \;

# And we'll set a trap:
trap "conf_cleanExit" SIGINT SIGTERM SIGHUP EXIT

